@@ -1,11 +1,42 @@
-/* Copyright (C) 1998, 1999, 2000, 2001  Free Software Foundation
+/* BufferedReader.java
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005
+     Free Software Foundation, Inc.
 
-   This file is part of libgcj.
+This file is part of GNU Classpath.
 
-This software is copyrighted work licensed under the terms of the
-Libgcj License.  Please consult the file "LIBGCJ_LICENSE" for
-details.  */
+GNU Classpath is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2, or (at your option)
+any later version.
  
+GNU Classpath is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with GNU Classpath; see the file COPYING.  If not, write to the
+Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+02111-1307 USA.
+
+Linking this library statically or dynamically with other modules is
+making a combined work based on this library.  Thus, the terms and
+conditions of the GNU General Public License cover the whole
+combination.
+
+As a special exception, the copyright holders of this library give you
+permission to link this library with independent modules to produce an
+executable, regardless of the license terms of these independent
+modules, and to copy and distribute the resulting executable under
+terms of your choice, provided that you also meet, for each linked
+independent module, the terms and conditions of the license of that
+module.  An independent module is a module which is not derived from
+or based on this library.  If you modify this library, you may extend
+this exception to your version of the library, but you are not
+obligated to do so.  If you do not wish to do so, delete this
+exception statement from your version. */
+
+
 package java.io;
 
 /* Written using "Java Class Libraries", 2nd edition, plus online
@@ -14,19 +45,19 @@ package java.io;
  */
 
 /**
-  * This subclass of <code>FilterReader</code> buffers input from an 
-  * underlying implementation to provide a possibly more efficient read
-  * mechanism.  It maintains the buffer and buffer state in instance 
-  * variables that are available to subclasses.  The default buffer size
-  * of 512 chars can be overridden by the creator of the stream.
-  * <p>
-  * This class also implements mark/reset functionality.  It is capable
-  * of remembering any number of input chars, to the limits of
-  * system memory or the size of <code>Integer.MAX_VALUE</code>
-  *
-  * @author Per Bothner <bothner@cygnus.com>
-  * @author Aaron M. Renn <arenn@urbanophile.com>
-  */
+ * This subclass of <code>FilterReader</code> buffers input from an 
+ * underlying implementation to provide a possibly more efficient read
+ * mechanism.  It maintains the buffer and buffer state in instance 
+ * variables that are available to subclasses.  The default buffer size
+ * of 8192 chars can be overridden by the creator of the stream.
+ * <p>
+ * This class also implements mark/reset functionality.  It is capable
+ * of remembering any number of input chars, to the limits of
+ * system memory or the size of <code>Integer.MAX_VALUE</code>
+ *
+ * @author Per Bothner (bothner@cygnus.com)
+ * @author Aaron M. Renn (arenn@urbanophile.com)
+ */
 public class BufferedReader extends Reader
 {
   Reader in;
@@ -53,37 +84,51 @@ public class BufferedReader extends Reader
      guaranteed to be >= the read-limit requested in the call to mark. */
   int markPos = -1;
 
+  // The JCL book specifies the default buffer size as 8K characters.
+  // This is package-private because it is used by LineNumberReader.
+  static final int DEFAULT_BUFFER_SIZE = 8192;
+
+  /**
+   * The line buffer for <code>readLine</code>.
+   */
+  private StringBuffer sbuf = null;
+
   /**
     * Create a new <code>BufferedReader</code> that will read from the 
-    * specified subordinate stream with a default buffer size of 4096 chars.
+    * specified subordinate stream with a default buffer size of 8192 chars.
     *
     * @param in The subordinate stream to read from
     */
   public BufferedReader(Reader in)
   {
-    this(in, 8192);
+    this(in, DEFAULT_BUFFER_SIZE);
   }
 
   /**
-    * Create a new <code>BufferedReader</code> that will read from the 
-    * specified subordinate stream with a buffer size that is specified by the 
-    * caller.
-    *
-    * @param in The subordinate stream to read from
-    * @param bufsize The buffer size to use
-    */
+   * Create a new <code>BufferedReader</code> that will read from the 
+   * specified subordinate stream with a buffer size that is specified by the 
+   * caller.
+   *
+   * @param in The subordinate stream to read from
+   * @param size The buffer size to use
+   *
+   * @exception IllegalArgumentException if size &lt;= 0
+   */
   public BufferedReader(Reader in, int size)
   {
     super(in.lock);
+    if (size <= 0)
+      throw new IllegalArgumentException("Illegal buffer size: " + size);
     this.in = in;
     buffer = new char[size];
   }
 
   /**
-    * This method closes the stream 
-    *
-    * @exception IOException If an error occurs
-    */
+   * This method closes the underlying stream and frees any associated
+   * resources.
+   *
+   * @exception IOException If an error occurs
+   */
   public void close() throws IOException
   {
     synchronized (lock)
@@ -96,38 +141,42 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * Returns <code>true</code> to indicate that this class supports mark/reset 
-    * functionality.
-    *
-    * @return <code>true</code>
-    */
+   * Returns <code>true</code> to indicate that this class supports mark/reset 
+   * functionality.
+   *
+   * @return <code>true</code>
+   */
   public boolean markSupported()
   {
     return true;
   }
 
   /**
-    * Mark a position in the input to which the stream can be
-    * "reset" by calling the <code>reset()</code> method.  The parameter
-    * <code>readlimit</code> is the number of chars that can be read from the 
-    * stream after setting the mark before the mark becomes invalid.  For
-    * example, if <code>mark()</code> is called with a read limit of 10, then 
-    * when 11 chars of data are read from the stream before the 
-    * <code>reset()</code> method is called, then the mark is invalid and the 
-    * stream object instance is not required to remember the mark.
-    * <p>
-    * Note that the number of chars that can be remembered by this method
-    * can be greater than the size of the internal read buffer.  It is also
-    * not dependent on the subordinate stream supporting mark/reset
-    * functionality.
-    *
-    * @param readlimit The number of chars that can be read before the mark 
-    *        becomes invalid
-    *
-    * @exception IOException If an error occurs
-    */
+   * Mark a position in the input to which the stream can be
+   * "reset" by calling the <code>reset()</code> method.  The parameter
+   * <code>readLimit</code> is the number of chars that can be read from the 
+   * stream after setting the mark before the mark becomes invalid.  For
+   * example, if <code>mark()</code> is called with a read limit of 10, then 
+   * when 11 chars of data are read from the stream before the 
+   * <code>reset()</code> method is called, then the mark is invalid and the 
+   * stream object instance is not required to remember the mark.
+   * <p>
+   * Note that the number of chars that can be remembered by this method
+   * can be greater than the size of the internal read buffer.  It is also
+   * not dependent on the subordinate stream supporting mark/reset
+   * functionality.
+   *
+   * @param readLimit The number of chars that can be read before the mark 
+   *        becomes invalid
+   *
+   * @exception IOException If an error occurs
+   * @exception IllegalArgumentException if readLimit is negative.
+   */
   public void mark(int readLimit) throws IOException
   {
+    if (readLimit < 0)
+      throw new IllegalArgumentException("Read-ahead limit is negative");
+
     synchronized (lock)
       {
 	checkStatus();
@@ -173,16 +222,16 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * Reset the stream to the point where the <code>mark()</code> method
-    * was called.  Any chars that were read after the mark point was set will
-    * be re-read during subsequent reads.
-    * <p>
-    * This method will throw an IOException if the number of chars read from
-    * the stream since the call to <code>mark()</code> exceeds the mark limit
-    * passed when establishing the mark.
-    *
-    * @exception IOException If an error occurs;
-    */
+   * Reset the stream to the point where the <code>mark()</code> method
+   * was called.  Any chars that were read after the mark point was set will
+   * be re-read during subsequent reads.
+   * <p>
+   * This method will throw an IOException if the number of chars read from
+   * the stream since the call to <code>mark()</code> exceeds the mark limit
+   * passed when establishing the mark.
+   *
+   * @exception IOException If an error occurs;
+   */
   public void reset() throws IOException
   {
     synchronized (lock)
@@ -205,14 +254,15 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * This method determines whether or not a stream is ready to be read.  If
-    * This method returns <code>false</code> then this stream could (but is
-    * not guaranteed to) block on the next read attempt.
-    *
-    * @return <code>true</code> if this stream is ready to be read, <code>false</code> otherwise
-    *
-    * @exception IOException If an error occurs
-    */
+   * This method determines whether or not a stream is ready to be read.  If
+   * this method returns <code>false</code> then this stream could (but is
+   * not guaranteed to) block on the next read attempt.
+   *
+   * @return <code>true</code> if this stream is ready to be read, 
+   * <code>false</code> otherwise
+   *
+   * @exception IOException If an error occurs
+   */
   public boolean ready() throws IOException
   {
     synchronized (lock)
@@ -223,25 +273,31 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * This method read chars from a stream and stores them into a caller
-    * supplied buffer.  It starts storing the data at index <code>offset</code> into
-    * the buffer and attempts to read <code>len</code> chars.  This method can
-    * return before reading the number of chars requested.  The actual number
-    * of chars read is returned as an int.  A -1 is returned to indicate the
-    * end of the stream.
-    * <p>
-    * This method will block until some data can be read.
-    *
-    * @param buf The array into which the chars read should be stored
-    * @param offset The offset into the array to start storing chars
-    * @param count The requested number of chars to read
-    *
-    * @return The actual number of chars read, or -1 if end of stream.
-    *
-    * @exception IOException If an error occurs.
-    */
+   * This method read chars from a stream and stores them into a caller
+   * supplied buffer.  It starts storing the data at index 
+   * <code>offset</code> into
+   * the buffer and attempts to read <code>len</code> chars.  This method can
+   * return before reading the number of chars requested.  The actual number
+   * of chars read is returned as an int.  A -1 is returned to indicate the
+   * end of the stream.
+   * <p>
+   * This method will block until some data can be read.
+   *
+   * @param buf The array into which the chars read should be stored
+   * @param offset The offset into the array to start storing chars
+   * @param count The requested number of chars to read
+   *
+   * @return The actual number of chars read, or -1 if end of stream.
+   *
+   * @exception IOException If an error occurs.
+   * @exception IndexOutOfBoundsException If offset and count are not
+   * valid regarding buf.
+   */
   public int read(char[] buf, int offset, int count) throws IOException
   {
+    if (offset < 0 || offset + count > buf.length || count < 0)
+      throw new IndexOutOfBoundsException();
+
     synchronized (lock)
       {
 	checkStatus();
@@ -321,7 +377,12 @@ public class BufferedReader extends Reader
     if (retAtEndOfBuffer && buffer[pos] == '\n')
       {
 	--count;
-	pos++;
+	// If the mark was set to the location of the \n, then we
+	// must change it to fully pretend that the \n does not
+	// exist.
+	if (markPos == pos)
+	  ++markPos;
+	++pos;
       }
 
     return count;
@@ -355,16 +416,16 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * This method reads a single line of text from the input stream, returning
-    * it as a <code>String</code>.  A line is terminated by "\n", a "\r", or
-    * an "\r\n" sequence.  The system dependent line separator is not used.
-    * The line termination characters are not returned in the resulting
-    * <code>String</code>.
-    * 
-    * @return The line of text read, or <code>null</code> if end of stream.
-    * 
-    * @exception IOException If an error occurs
-    */
+   * This method reads a single line of text from the input stream, returning
+   * it as a <code>String</code>.  A line is terminated by "\n", a "\r", or
+   * an "\r\n" sequence.  The system dependent line separator is not used.
+   * The line termination characters are not returned in the resulting
+   * <code>String</code>.
+   * 
+   * @return The line of text read, or <code>null</code> if end of stream.
+   * 
+   * @exception IOException If an error occurs
+   */
   public String readLine() throws IOException
   {
     checkStatus();
@@ -383,7 +444,7 @@ public class BufferedReader extends Reader
     int i = lineEnd(limit);
     if (i < limit)
       {
-	String str = new String(buffer, pos, i - pos);
+	String str = String.valueOf(buffer, pos, i - pos);
 	pos = i + 1;
 	// If the last char in the buffer is a '\r', we must remember
 	// to check if the next char to be read after the buffer is refilled
@@ -394,7 +455,10 @@ public class BufferedReader extends Reader
 	    pos++;
 	return str;
       }
-    StringBuffer sbuf = new StringBuffer(200);
+    if (sbuf == null)
+      sbuf = new StringBuffer(200);
+    else
+      sbuf.setLength(0);
     sbuf.append(buffer, pos, i - pos);
     pos = i;
     // We only want to return null when no characters were read before
@@ -404,12 +468,19 @@ public class BufferedReader extends Reader
     boolean eof = false;
     for (;;)
       {
-	int ch = read();
-	if (ch < 0)
+	// readLine should block. So we must not return until a -1 is reached.
+	if (pos >= limit)
 	  {
-	    eof = true;
-	    break;
+	    // here count == 0 isn't sufficient to give a failure.
+	    int count = fill();
+	    if (count < 0)
+	      {
+		eof = true;
+		break;
+	      }
+	    continue;
 	  }
+	int ch = buffer[pos++];
 	if (ch == '\n' || ch == '\r')
 	  {
 	    // Check here if a '\r' was the last char in the buffer; if so,
@@ -429,25 +500,29 @@ public class BufferedReader extends Reader
   }
 
   /**
-    * This method skips the specified number of chars in the stream.  It
-    * returns the actual number of chars skipped, which may be less than the
-    * requested amount.
-    * <p>
-    * This method first discards chars in the buffer, then calls the
-    * <code>skip</code> method on the underlying stream to skip the remaining chars.
-    *
-    * @param num_chars The requested number of chars to skip
-    *
-    * @return The actual number of chars skipped.
-    *
-    * @exception IOException If an error occurs
-    */
+   * This method skips the specified number of chars in the stream.  It
+   * returns the actual number of chars skipped, which may be less than the
+   * requested amount.
+   * <p>
+   * This method first discards chars in the buffer, then calls the
+   * <code>skip</code> method on the underlying stream to skip the 
+   * remaining chars.
+   *
+   * @param count The requested number of chars to skip
+   *
+   * @return The actual number of chars skipped.
+   *
+   * @exception IOException If an error occurs.
+   * @exception IllegalArgumentException If count is negative.
+   */
   public long skip(long count) throws IOException
   {
     synchronized (lock)
       {
 	checkStatus();
-	if (count <= 0)
+	if (count < 0)
+	  throw new IllegalArgumentException("skip value is negative");
+	if (count == 0)
 	  return 0;
 	// Yet again, we need to handle the special case of a readLine
 	// that has a '\r' at the end of the buffer.  In this case, we need
@@ -458,12 +533,13 @@ public class BufferedReader extends Reader
 	// skip the '\n' for us).  By doing this, we'll have to back up pos.
 	// That's easier than trying to keep track of whether we've skipped
 	// one element or not.
-	int ch;
 	if (pos > limit)
-	  if ((ch = read()) < 0)
-	    return 0;
-	  else
-	    --pos; 
+	  {
+	    if (read() < 0)
+	      return 0;
+	    else
+	      --pos; 
+	  }
 
 	int avail = limit - pos;
 
