@@ -1,5 +1,5 @@
 /* BitSet.java -- A vector of bits.
-   Copyright (C) 1998, 1999, 2000, 2001  Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2001, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -61,8 +61,8 @@ import java.io.Serializable;
  * undefined.
  *
  * @author Jochen Hoenicke
- * @author Tom Tromey <tromey@cygnus.com>
- * @author Eric Blake <ebb9@email.byu.edu>
+ * @author Tom Tromey (tromey@cygnus.com)
+ * @author Eric Blake (ebb9@email.byu.edu)
  * @status updated to 1.4
  */
 public class BitSet implements Cloneable, Serializable
@@ -133,7 +133,8 @@ public class BitSet implements Cloneable, Serializable
    * Performs the logical AND operation on this bit set and the
    * complement of the given <code>set</code>.  This means it
    * selects every element in the first set, that isn't in the
-   * second set.  The result is stored into this bit set.
+   * second set.  The result is stored into this bit set and is
+   * effectively the set difference of the two.
    *
    * @param set the second bit set
    * @throws NullPointerException if set is null
@@ -210,7 +211,8 @@ public class BitSet implements Cloneable, Serializable
    *
    * @param from the start range (inclusive)
    * @param to the end range (exclusive)
-   * @throws IndexOutOfBoundsException if from &lt; 0 || from &gt; to
+   * @throws IndexOutOfBoundsException if from &lt; 0 || to &lt; 0 ||
+   *         from &gt; to
    * @since 1.4
    */
   public void clear(int from, int to)
@@ -304,7 +306,8 @@ public class BitSet implements Cloneable, Serializable
    *
    * @param from the low index (inclusive)
    * @param to the high index (exclusive)
-   * @throws IndexOutOfBoundsException if from &gt; to || from &lt; 0
+   * @throws IndexOutOfBoundsException if from &gt; to || from &lt; 0 ||
+   *         to &lt; 0
    * @since 1.4
    */
   public void flip(int from, int to)
@@ -352,7 +355,8 @@ public class BitSet implements Cloneable, Serializable
    *
    * @param from the low index (inclusive)
    * @param to the high index (exclusive)
-   * @throws IndexOutOfBoundsException if from &gt; to || from &lt; 0
+   * @throws IndexOutOfBoundsException if from &gt; to || from &lt; 0 ||
+   *         to &lt; 0
    * @since 1.4
    */
   public BitSet get(int from, int to)
@@ -376,7 +380,7 @@ public class BitSet implements Cloneable, Serializable
       }
 
     int len = Math.min(hi_offset, bits.length - 1);
-    int reverse = ~lo_bit;
+    int reverse = 64 - lo_bit;
     int i;
     for (i = 0; lo_offset < len; lo_offset++, i++)
       bs.bits[i] = ((bits[lo_offset] >>> lo_bit)
@@ -618,7 +622,8 @@ public class BitSet implements Cloneable, Serializable
    *
    * @param from the start range (inclusive)
    * @param to the end range (exclusive)
-   * @throws IndexOutOfBoundsException if from &lt; 0 || from &gt; to
+   * @throws IndexOutOfBoundsException if from &lt; 0 || from &gt; to ||
+   *         to &lt; 0
    * @since 1.4
    */
   public void set(int from, int to)
@@ -649,7 +654,8 @@ public class BitSet implements Cloneable, Serializable
    * @param from the start range (inclusive)
    * @param to the end range (exclusive)
    * @param value the value to set it to
-   * @throws IndexOutOfBoundsException if from &lt; 0 || from &gt; to
+   * @throws IndexOutOfBoundsException if from &lt; 0 || from &gt; to ||
+   *         to &lt; 0
    * @since 1.4
    */
   public void set(int from, int to, boolean value)
@@ -726,7 +732,7 @@ public class BitSet implements Cloneable, Serializable
    *
    * @param lastElt the size needed for the bits array
    */
-  private final void ensure(int lastElt)
+  private void ensure(int lastElt)
   {
     if (lastElt >= bits.length)
       {
